/**
    CS-11 Asn 2
    indp.h
    Purpose: header for indp.h

    @author Max Schroetter
    @version 0.1 13.12.17
    Fix-Later Comments to be fixed
*/
#include "ndpCache.h"
#include "protocols/ip.h"
#include <vector>
#include <map>

#define INDP_GID 1001
#define INDP_SPOOFEDREDIRECT_SID 1
#define INDP_FLOOD_SID 2
#define INDP_ROUTERKILL_SID 3
#define INDP_NONRA_SID 4
#define INDP_NEWRA_SID 5
#define INDP_CHRAFLAG_SID 6
#define INDP_CHRAPREFIX_SID 7
#define INDP_NEWDAD_SID 8
#define INDP_NEWHOST_SID 9
#define INDP_DADCONFLICT_SID 10
#define INDP_DADSPOOFED_SID 11
#define INDP_UNSOLICAD_SID 12
#define INDP_MLDFLOOD 13
#define INDP_MLDNONROUTER 14
#define INDP_POSSIBLENSPOISON 15
#define INDP_MLDQryNR 16
#define INDP_MLDNONRA 17


struct host_key 
{
    uint64_t uip6addr;
    uint64_t lip6addr;

    bool operator==(const host_key &other) const {
        return (uip6addr == other.uip6addr && lip6addr == other.lip6addr);
    }
};

struct host_key_hasher
{
  std::size_t operator()(const host_key &k) const
  {
    using std::size_t;
    using std::hash;
    using std::string;

    return ((hash<uint64_t>()(k.uip6addr)
             ^ (hash<uint64_t>()(k.lip6addr) << 1)) >> 1);
  }
};

struct mac_entry
{
	uint8_t mac_addr[6];
};

struct router_entry
{
	uint8_t mac_addr[6];
	ip::snort_in6_addr src_addr;
	uint8_t flags;
	uint8_t prefix_flags;
	ip::snort_in6_addr prefix;


};
struct host_entry
{
    uint8_t mac_addr[6];
	ip::snort_in6_addr ip6_addr;
	int advertise_count;
    time_t last_ns;
    bool check[3];
    time_t last_na;
};

typedef std::vector<mac_entry> MacList;
typedef std::vector<SfCidr> PrefixList;
typedef uint64_t Mac_Id;
typedef NdpCache<host_key,host_entry,host_key_hasher> HostCache;
typedef NdpCache<Mac_Id,router_entry,std::hash<unsigned long long>> RouterCache;
typedef std::map<Mac_Id,router_entry> RouterList; // Fix-Later to be removed
typedef std::map<Mac_Id,host_entry> HostList; //Fix-Later to be removed

struct indpconfig {
    long topology_size;
    long flood_window_time;
    long flood_window_count;
    MacList allowed_routers;
    bool use_ipv6;
};

class indpstate {
    public:
    RouterCache* routers;
    HostCache* confirmed;
    HostCache* unconfirmed;
    indpconfig config;
    indpstate(indpconfig config_):config(config_){
        routers=new RouterCache(config_.topology_size); 
        confirmed=new HostCache(config_.topology_size);
        unconfirmed=new HostCache(config_.topology_size); 
    }
};
