#include <list>
#include <unordered_map>

using namespace std;


template <class K, class V, class Hash> class NdpCache
{
private:
    unordered_map<K, typename list< pair<K,V>>::iterator , Hash> item_map;
    size_t cache_size;
public:
    list< pair<K,V> > item_list;
    NdpCache(size_t cache_size_):cache_size(cache_size_)
    {
        ;
    };

    void put(const K &key, const V &val)
    {
        typename unordered_map<K, typename list< pair<K,V>>::iterator, Hash>::iterator it =
                    item_map.find(key);
        if(it != item_map.end())
        {
            item_list.erase(it->second);
            item_map.erase(it);
        }
        item_list.push_front(make_pair(key,val));
        item_map.insert(make_pair(key, item_list.begin()));
        //manage Cache size
        while(item_map.size()>cache_size)
        {
            auto last_it = item_list.end();
            last_it --;
            item_map.erase(last_it->first);
            item_list.pop_back();
        }
    };

    void remove(const K &key)
    {
        typename unordered_map<K, typename list< pair<K,V>>::iterator, Hash>::iterator it =
                    item_map.find(key);
        if(it != item_map.end())
        {
            item_list.erase(it->second);
            item_map.erase(it);
        }
    }

    bool exist(const K &key)
    {
        return (item_map.count(key)>0);
    };

    V get(const K &key)
    {
        typename unordered_map<K, typename list< pair<K,V>>::iterator, Hash>::iterator it =
                    item_map.find(key);
        item_list.splice(item_list.begin(), item_list, it->second);
        return (it->second->second);
    };
    V* getp(const K &key)
    {
        typename unordered_map<K, typename list< pair<K,V>>::iterator, Hash>::iterator it =
                    item_map.find(key);
        item_list.splice(item_list.begin(), item_list, it->second);
        return &(it->second->second);
    };
};
