#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

//#include "detection/detection_defines.h"
#include "framework/ips_option.h"
#include "framework/module.h"
#include "framework/range.h"
#include "hash/hashfcn.h"
#include "profiler/profiler.h"
#include "protocols/packet.h"
#include "protocols/tcp.h"
#include "log/messages.h"
#include "sfip/sf_cidr.h"

#include <string>
#include "protocols/protocol_ids.h"

static const char* s_name = "ptest";
static const char* s_help = "rule for ip version detection";

static THREAD_LOCAL ProfileStats testPerfStats;

//-------------------------------------------------------------------------
// option
//-------------------------------------------------------------------------

class TestOption : public IpsOption
{
public:
    TestOption(SfCidr a) : IpsOption(s_name)
    {
        this-> prefix = a;
    }

    bool operator==(const IpsOption&) const override;

    IpsOption::EvalStatus eval(Cursor&, Packet*) override;

private:
    SfCidr prefix;
};


bool TestOption::operator==(const IpsOption& ips) const
{
    if (strcmp(s_name, ips.get_name()))
    {
        return false;
    }
    return false;
}

IpsOption::EvalStatus TestOption::eval(Cursor&, Packet* p)
{
    Profile profile(testPerfStats);
    IpsOption::EvalStatus rval = NO_MATCH;
//    Test section
    if (prefix.contains(p->ptrs.ip_api.get_src()) == SfIpRet::SFIP_CONTAINS)
        rval = MATCH;
    return rval;
}

//-------------------------------------------------------------------------
// module
//-------------------------------------------------------------------------

static const Parameter s_params[] =
{
    {
        "~", Parameter::PT_ADDR, nullptr, nullptr,
        "IPv6 Prefix in CIDR"
    },
    { nullptr, Parameter::PT_MAX, nullptr, nullptr, nullptr }
};

class TestModule : public Module
{
public:
    TestModule() : Module(s_name, s_help, s_params) { }

    bool set(const char*, Value&, SnortConfig*) override;

    ProfileStats* get_profile() const override
    {
        return &testPerfStats;
    }

    SfCidr prefix;
};


bool TestModule::set(const char*, Value& v, SnortConfig*)
{
      //  LogMessage(v.get_string());
        v.get_addr(prefix);
        prefix.set_bits(64);
      //  if(!prefix.set(v.get_string())){
      //      return false;
      //  }
        LogMessage(prefix.ntoa());
        LogMessage("%d", prefix.get_bits());
        return true;
}

//-------------------------------------------------------------------------
// api methods
//-------------------------------------------------------------------------

static Module* mod_ctor()
{
    return new TestModule;
}

static void mod_dtor(Module* m)
{
    delete m;
}

static IpsOption* test_ctor(Module* p, OptTreeNode*)
{
    TestModule* m = (TestModule*)p;
    return new TestOption(m->prefix);
}

static void test_dtor(IpsOption* p)
{
    delete p;
}

static const IpsApi test_api =
{
    {
        PT_IPS_OPTION,
        sizeof(IpsApi),
        IPSAPI_VERSION,
        0,
        API_RESERVED,
        API_OPTIONS,
        s_name,
        s_help,
        mod_ctor,
        mod_dtor
    },
    OPT_TYPE_DETECTION,
    1, PROTO_BIT__IP,
    nullptr, // pinit
    nullptr, // pterm
    nullptr, // tinit
    nullptr, // tterm
    test_ctor,
    test_dtor,
    nullptr
};

SO_PUBLIC const BaseApi* snort_plugins[] =
{
    &test_api.base,
    nullptr
};

